@@ -3,6 +3,7 @@
  * GEOS - Geometry Engine Open Source
  * http://geos.osgeo.org
  *
+ * Copyright (C) 2020 Sandro Santilli <strk@kbt.io>
  * Copyright (C) 2001-2002 Vivid Solutions Inc.
  *
  * This is free software; you can redistribute and/or modify it under
@@ -18,9 +19,13 @@
 
 
 #include <geos/geom/prep/PreparedLineString.h>
+#include <geos/geom/prep/PreparedLineStringDistance.h>
 #include <geos/geom/prep/PreparedLineStringIntersects.h>
+#include <geos/geom/prep/PreparedLineStringNearestPoints.h>
+#include <geos/geom/GeometryFactory.h>
 #include <geos/noding/SegmentStringUtil.h>
 #include <geos/noding/FastSegmentSetIntersectionFinder.h>
+#include <geos/operation/distance/IndexedFacetDistance.h>
 
 namespace geos {
 namespace geom { // geos.geom
@@ -61,6 +66,37 @@ PreparedLineString::intersects(const geom::Geometry* g) const
     return PreparedLineStringIntersects::intersects(prep, g);
 }
 
+/* public */
+operation::distance::IndexedFacetDistance*
+PreparedLineString::
+getIndexedFacetDistance() const
+{
+    if(! indexedDistance ) {
+        indexedDistance.reset(new operation::distance::IndexedFacetDistance(&getGeometry()));
+    }
+    return indexedDistance.get();
+}
+
+
+std::unique_ptr<geom::CoordinateSequence>
+PreparedLineString::nearestPoints(const geom::Geometry* g) const
+{
+    return PreparedLineStringNearestPoints::nearestPoints(*this, g);
+}
+
+double
+PreparedLineString::distance(const geom::Geometry* g) const
+{
+    return PreparedLineStringDistance::distance(*this, g);
+}
+
+bool
+PreparedLineString::isWithinDistance(const geom::Geometry* g, double d) const
+{
+    return PreparedLineStringDistance(*this).isWithinDistance(g, d);
+}
+
+
 } // namespace geos.geom.prep
 } // namespace geos.geom
 } // namespace geos
