@@ -32,7 +32,6 @@
 #define GEOS_DEBUG 0
 #endif
 
-using namespace std;
 using namespace geos::geom;
 
 namespace geos {
@@ -43,37 +42,32 @@ NodeMap::NodeMap(const NodeFactory& newNodeFact)
     nodeFact(newNodeFact)
 {
 #if GEOS_DEBUG
-    cerr << "[" << this << "] NodeMap::NodeMap" << endl;
+    std::cerr << "[" << this << "] NodeMap::NodeMap" << std::endl;
 #endif
 }
 
-NodeMap::~NodeMap()
-{
-    for(auto& it: nodeMap) {
-        delete it.second;
-    }
-}
+NodeMap::~NodeMap() = default;
 
 Node*
 NodeMap::addNode(const Coordinate& coord)
 {
 #if GEOS_DEBUG
-    cerr << "[" << this << "] NodeMap::addNode(" << coord.toString() << ")";
+    std::cerr << "[" << this << "] NodeMap::addNode(" << coord.toString() << ")";
 #endif
     Node* node = find(coord);
     if(node == nullptr) {
 #if GEOS_DEBUG
-        cerr << " is new" << endl;
+        std::cerr << " is new" << std::endl;
 #endif
         node = nodeFact.createNode(coord);
         Coordinate* c = const_cast<Coordinate*>(
                             &(node->getCoordinate()));
-        nodeMap[c] = node;
-        //nodeMap[const_cast<Coordinate *>(&coord)]=node;
+        nodeMap[c] = std::unique_ptr<Node>(node);
+        node = nodeMap[c].get();
     }
     else {
 #if GEOS_DEBUG
-        cerr << " already found (" << node->getCoordinate().toString() << ") - adding Z" << endl;
+        std::cerr << " already found (" << node->getCoordinate().toString() << ") - adding Z" << std::endl;
 #endif
         node->addZ(coord.z);
     }
@@ -88,21 +82,21 @@ NodeMap::addNode(Node* n)
     assert(n);
 
 #if GEOS_DEBUG
-    cerr << "[" << this << "] NodeMap::addNode(" << n->print() << ")";
+    std::cerr << "[" << this << "] NodeMap::addNode(" << n->print() << ")";
 #endif
     Coordinate* c = const_cast<Coordinate*>(&n->getCoordinate());
     Node* node = find(*c);
     if(node == nullptr) {
 #if GEOS_DEBUG
-        cerr << " is new" << endl;
+        std::cerr << " is new" << std::endl;
 #endif
-        nodeMap[c] = n;
-        return n;
+        nodeMap[c] = std::unique_ptr<Node>(n);
+        return nodeMap[c].get();
     }
 #if GEOS_DEBUG
     else {
-        cerr << " found already, merging label" << endl;
-        const vector<double>& zvals = n->getZ();
+        std::cerr << " found already, merging label" << std::endl;
+        const std::vector<double>& zvals = n->getZ();
         for(unsigned int i = 0; i < zvals.size(); i++) {
             node->addZ(zvals[i]);
         }
@@ -121,6 +115,13 @@ NodeMap::add(EdgeEnd* e)
     n->add(e);
 }
 
+void
+NodeMap::add(std::unique_ptr<EdgeEnd>&& e)
+{
+    add(e.get());
+    e.release();
+}
+
 /*
  * @return the node if found; null otherwise
  */
@@ -129,33 +130,33 @@ NodeMap::find(const Coordinate& coord) const
 {
     Coordinate* c = const_cast<Coordinate*>(&coord);
 
-    NodeMap::const_iterator found = nodeMap.find(c);
+    const auto& found = nodeMap.find(c);
 
     if(found == nodeMap.end()) {
         return nullptr;
     }
     else {
-        return found->second;
+        return found->second.get();
     }
 }
 
 void
-NodeMap::getBoundaryNodes(int geomIndex, vector<Node*>& bdyNodes) const
+NodeMap::getBoundaryNodes(uint8_t geomIndex, std::vector<Node*>& bdyNodes) const
 {
-    for(auto& it: nodeMap) {
-        Node* node = it.second;
+    for(const auto& it: nodeMap) {
+        Node* node = it.second.get();
         if(node->getLabel().getLocation(geomIndex) == Location::BOUNDARY) {
             bdyNodes.push_back(node);
         }
     }
 }
 
-string
+std::string
 NodeMap::print() const
 {
-    string out = "";
-    for(auto& it: nodeMap) {
-        Node* node = it.second;
+    std::string out = "";
+    for(const auto& it: nodeMap) {
+        const Node* node = it.second.get();
         out += node->print();
     }
     return out;
