@@ -16,13 +16,15 @@
  *
  **********************************************************************/
 
-#include <geos/simplify/TaggedLineStringSimplifier.h>
-#include <geos/simplify/LineSegmentIndex.h>
 #include <geos/algorithm/LineIntersector.h>
-#include <geos/simplify/TaggedLineString.h>
-#include <geos/simplify/TaggedLineSegment.h>
+#include <geos/algorithm/Orientation.h>
 #include <geos/geom/CoordinateSequence.h>
 #include <geos/geom/LineString.h>
+#include <geos/simplify/ComponentJumpChecker.h>
+#include <geos/simplify/LineSegmentIndex.h>
+#include <geos/simplify/TaggedLineStringSimplifier.h>
+#include <geos/simplify/TaggedLineString.h>
+#include <geos/simplify/TaggedLineSegment.h>
 #include <geos/util.h>
 
 #include <algorithm>
@@ -37,9 +39,11 @@
 #include <iostream>
 #endif
 
-using namespace geos::geom;
-using std::unique_ptr;
-using std::vector;
+using geos::geom::LineSegment;
+using geos::geom::Coordinate;
+using geos::geom::LineString;
+using geos::algorithm::LineIntersector;
+using geos::algorithm::Orientation;
 
 namespace geos {
 namespace simplify { // geos::simplify
@@ -47,20 +51,20 @@ namespace simplify { // geos::simplify
 /*public*/
 TaggedLineStringSimplifier::TaggedLineStringSimplifier(
     LineSegmentIndex* nInputIndex,
-    LineSegmentIndex* nOutputIndex)
-    :
-    inputIndex(nInputIndex),
-    outputIndex(nOutputIndex),
-    li(new algorithm::LineIntersector()),
-    line(nullptr),
-    linePts(nullptr),
-    distanceTolerance(0.0)
-{
-}
+    LineSegmentIndex* nOutputIndex,
+    const ComponentJumpChecker* crossChecker)
+    : inputIndex(nInputIndex)
+    , outputIndex(nOutputIndex)
+    , jumpChecker(crossChecker)
+    , li(new LineIntersector())
+    , line(nullptr)
+    , linePts(nullptr)
+{}
+
 
 /*public*/
 void
-TaggedLineStringSimplifier::simplify(TaggedLineString* nLine)
+TaggedLineStringSimplifier::simplify(TaggedLineString* nLine, double distanceTolerance)
 {
     assert(nLine);
     line = nLine;
@@ -78,17 +82,17 @@ TaggedLineStringSimplifier::simplify(TaggedLineString* nLine)
     if(linePts->isEmpty()) {
         return;
     }
-    simplifySection(0, linePts->size() - 1, 0);
+    simplifySection(0, linePts->size() - 1, 0, distanceTolerance);
 
-    if(!line->getPreserveEndpoint() && linePts->isRing()) {
-        simplifyRingEndpoint();
+    if(line->isRing() && linePts->isRing()) {
+        simplifyRingEndpoint(distanceTolerance);
     }
 }
 
 /*private*/
 void
 TaggedLineStringSimplifier::simplifySection(std::size_t i,
-        std::size_t j, std::size_t depth)
+        std::size_t j, std::size_t depth, double distanceTolerance)
 {
     depth += 1;
 
@@ -145,11 +149,10 @@ TaggedLineStringSimplifier::simplifySection(std::size_t i,
         isValidToSimplify = false;
     }
 
-    // test if flattened section would cause intersection
-    LineSegment candidateSeg(linePts->getAt(i), linePts->getAt(j));
-
-    if(hasBadIntersection(line, i, j, candidateSeg)) {
-        isValidToSimplify = false;
+    if (isValidToSimplify) {
+        // test if flattened section would cause intersection or jump
+        LineSegment flatSeg(linePts->getAt(i), linePts->getAt(j));
+        isValidToSimplify = isTopologyValid(line, i, j, flatSeg);
     }
 
     if(isValidToSimplify) {
@@ -167,22 +170,23 @@ TaggedLineStringSimplifier::simplifySection(std::size_t i,
         return;
     }
 
-    simplifySection(i, furthestPtIndex, depth);
-    simplifySection(furthestPtIndex, j, depth);
+    simplifySection(i, furthestPtIndex, depth, distanceTolerance);
+    simplifySection(furthestPtIndex, j, depth, distanceTolerance);
 }
 
 /*private*/
 void
-TaggedLineStringSimplifier::simplifyRingEndpoint()
+TaggedLineStringSimplifier::simplifyRingEndpoint(double distanceTolerance)
 {
     if (line->getResultSize() > line->getMinimumSize()) {
-        const auto* firstSeg = line->getResultSegments().front();
-        const auto* lastSeg = line->getResultSegments().back();
+        const auto* firstSeg = static_cast<LineSegment*>(line->getResultSegments().front());
+        const auto* lastSeg = static_cast<LineSegment*>(line->getResultSegments().back());
 
-        LineSegment candidateSeg(lastSeg->p0, firstSeg->p1);
-        if (candidateSeg.distance(firstSeg->p0) <= distanceTolerance &&
-                ! hasBadIntersection(line, line->getSegments().size() - 2, 0, candidateSeg)) {
-            //auto newSeg = detail::make_unique<TaggedLineSegment>(candidateSeg.p0, candidateSeg.p1);
+        LineSegment simpSeg(lastSeg->p0, firstSeg->p1);
+        const Coordinate& endPt = firstSeg->p0;
+        if (simpSeg.distance(endPt) <= distanceTolerance &&
+            isTopologyValid(line, firstSeg, lastSeg, simpSeg))
+        {
             line->removeRingEndpoint();
         }
     }
@@ -197,39 +201,66 @@ TaggedLineStringSimplifier::flatten(std::size_t start, std::size_t end)
     const Coordinate& p1 = linePts->getAt(end);
     std::unique_ptr<TaggedLineSegment> newSeg(new TaggedLineSegment(p0, p1));
     // update the indexes
-    remove(line, start, end);
     outputIndex->add(newSeg.get());
+    remove(line, start, end);
     return newSeg;
 }
 
 /*private*/
 bool
-TaggedLineStringSimplifier::hasBadIntersection(
-    const TaggedLineString* parentLine,
-    const size_t excludeStart, const size_t excludeEnd,
-    const LineSegment& candidateSeg)
+TaggedLineStringSimplifier::isTopologyValid(
+    const TaggedLineString* lineIn,
+    std::size_t sectionStart, std::size_t sectionEnd,
+    const LineSegment& flatSeg)
 {
-    if(hasBadOutputIntersection(candidateSeg)) {
-        return true;
-    }
-
-    if(hasBadInputIntersection(parentLine, excludeStart, excludeEnd, candidateSeg)) {
-        return true;
-    }
-
-    return false;
+    if (hasOutputIntersection(flatSeg))
+        return false;
+    if (hasInputIntersection(lineIn, sectionStart, sectionEnd, flatSeg))
+        return false;
+    if (jumpChecker->hasJump(lineIn, sectionStart, sectionEnd, flatSeg))
+        return false;
+    return true;
 }
 
 /*private*/
 bool
-TaggedLineStringSimplifier::hasBadOutputIntersection(
-    const LineSegment& candidateSeg)
+TaggedLineStringSimplifier::isTopologyValid(
+    const TaggedLineString* lineIn,
+    const LineSegment* seg1, const LineSegment* seg2,
+    const LineSegment& flatSeg)
 {
-    std::unique_ptr< std::vector<LineSegment*> > querySegs =
-        outputIndex->query(&candidateSeg);
+    //-- if segments are already flat, topology is unchanged and so is valid
+    //-- (otherwise, output and/or input intersection test would report false positive)
+    if (isCollinear(seg1->p0, flatSeg))
+        return true;
+    if (hasOutputIntersection(flatSeg))
+        return false;
+    if (hasInputIntersection(flatSeg))
+        return false;
+    if (jumpChecker->hasJump(lineIn, seg1, seg2, flatSeg))
+        return false;
+    return true;
+}
+
+/*private*/
+bool
+TaggedLineStringSimplifier::isCollinear(
+    const Coordinate& pt,
+    const LineSegment& seg) const
+{
+    return Orientation::COLLINEAR == seg.orientationIndex(pt);
+}
+
+/*private*/
+bool
+TaggedLineStringSimplifier::hasOutputIntersection(
+    const LineSegment& flatSeg)
+{
+    //std::unique_ptr<std::vector<LineSegment*>>
+    auto querySegs = outputIndex->query(&flatSeg);
 
     for(const LineSegment* querySeg : *querySegs) {
-        if(hasInteriorIntersection(*querySeg, candidateSeg)) {
+        if(hasInvalidIntersection(*querySeg, flatSeg)) {
             return true;
         }
     }
@@ -239,27 +270,44 @@ TaggedLineStringSimplifier::hasBadOutputIntersection(
 
 /*private*/
 bool
-TaggedLineStringSimplifier::hasInteriorIntersection(
+TaggedLineStringSimplifier::hasInvalidIntersection(
     const LineSegment& seg0,
     const LineSegment& seg1) const
 {
+    if(seg0.equalsTopo(seg1))
+        return true;
     li->computeIntersection(seg0.p0, seg0.p1, seg1.p0, seg1.p1);
     return li->isInteriorIntersection();
 }
 
 /*private*/
 bool
-TaggedLineStringSimplifier::hasBadInputIntersection(
-    const TaggedLineString* parentLine,
-    const size_t excludeStart, const size_t excludeEnd,
-    const LineSegment& candidateSeg)
+TaggedLineStringSimplifier::hasInputIntersection(const LineSegment& flatSeg)
 {
-    const auto& foundSegs = inputIndex->query(&candidateSeg);
+    return hasInputIntersection(nullptr, 0, 0, flatSeg);
+}
 
-    for(const LineSegment* ls : *foundSegs) {
-        const TaggedLineSegment* foundSeg = static_cast<const TaggedLineSegment*>(ls);
+/*private*/
+bool
+TaggedLineStringSimplifier::hasInputIntersection(
+    const TaggedLineString* parentLine,
+    const std::size_t excludeStart, const std::size_t excludeEnd,
+    const LineSegment& flatSeg)
+{
+    const auto& querySegs = inputIndex->query(&flatSeg);
 
-        if(!isInLineSection(parentLine, excludeStart, excludeEnd, foundSeg) && hasInteriorIntersection(*foundSeg, candidateSeg)) {
+    for(const LineSegment* ls : *querySegs) {
+        const TaggedLineSegment* querySeg = static_cast<const TaggedLineSegment*>(ls);
+
+        if (hasInvalidIntersection(*ls, flatSeg)) {
+            /**
+             * Ignore the intersection if the intersecting segment is part of the section being collapsed
+             * to the candidate segment
+             */
+            if (parentLine != nullptr &&
+                isInLineSection(line, excludeStart, excludeEnd, querySeg)) {
+                continue;
+            }
             return true;
         }
     }
@@ -270,25 +318,25 @@ TaggedLineStringSimplifier::hasBadInputIntersection(
 /*static private*/
 bool
 TaggedLineStringSimplifier::isInLineSection(
-    const TaggedLineString* line,
-    const size_t excludeStart, const size_t excludeEnd,
+    const TaggedLineString* lineIn,
+    const std::size_t excludeStart, const std::size_t excludeEnd,
     const TaggedLineSegment* seg)
 {
     // not in this line
-    if(seg->getParent() != line->getParent()) {
+    if(seg->getParent() != lineIn->getParent()) {
         return false;
     }
 
     std::size_t segIndex = seg->getIndex();
     if (excludeStart <= excludeEnd) {
-      //-- section is contiguous
-      if (segIndex >= excludeStart && segIndex < excludeEnd)
-        return true;
+        //-- section is contiguous
+        if (segIndex >= excludeStart && segIndex < excludeEnd)
+            return true;
     }
     else {
-      //-- section wraps around the end of a ring
-      if (segIndex >= excludeStart || segIndex <= excludeEnd)
-      return true;
+        //-- section wraps around the end of a ring
+        if (segIndex >= excludeStart || segIndex <= excludeEnd)
+            return true;
     }
     return false;
 }
@@ -341,8 +389,8 @@ TaggedLineStringSimplifier::findFurthestPoint(
     }
     maxDistance = maxDist;
     return maxIndex;
-
 }
 
+
 } // namespace geos::simplify
 } // namespace geos
