@@ -16,7 +16,6 @@
 #include <geos/algorithm/Orientation.h>
 #include <geos/geom/Coordinate.h>
 #include <geos/geom/CoordinateSequence.h>
-#include <geos/geom/CoordinateArraySequence.h>
 #include <geos/geom/Envelope.h>
 #include <geos/geom/GeometryFactory.h>
 #include <geos/geom/Polygon.h>
@@ -35,7 +34,7 @@ namespace polygon {
 
 
 /* public */
-PolygonEarClipper::PolygonEarClipper(const std::vector<Coordinate>& polyShell)
+PolygonEarClipper::PolygonEarClipper(const geom::CoordinateSequence& polyShell)
     : vertex(polyShell)
     , vertexSize(polyShell.size()-1)
     , vertexFirst(0)
@@ -45,6 +44,7 @@ PolygonEarClipper::PolygonEarClipper(const std::vector<Coordinate>& polyShell)
     initCornerIndex();
 }
 
+
 /* private */
 std::vector<std::size_t>
 PolygonEarClipper::createNextLinks(std::size_t size) const
@@ -60,21 +60,12 @@ PolygonEarClipper::createNextLinks(std::size_t size) const
 
 /* public static */
 void
-PolygonEarClipper::triangulate(const std::vector<Coordinate>& polyShell, TriList<Tri>& triListResult)
+PolygonEarClipper::triangulate(const geom::CoordinateSequence& polyShell, TriList<Tri>& triListResult)
 {
     PolygonEarClipper clipper(polyShell);
     clipper.compute(triListResult);
 }
 
-/* public static */
-void
-PolygonEarClipper::triangulate(const CoordinateSequence& polyShellCs, TriList<Tri>& triListResult)
-{
-    std::vector<Coordinate> pts;
-    polyShellCs.toVector(pts);
-    PolygonEarClipper clipper(pts);
-    clipper.compute(triListResult);
-}
 
 /* public */
 void
@@ -151,7 +142,7 @@ PolygonEarClipper::isValidEar(std::size_t cornerIdx, const std::array<Coordinate
 {
     std::size_t intApexIndex = findIntersectingVertex(cornerIdx, corner);
     //--- no intersections found
-    if (intApexIndex == NO_VERTEX_INDEX)
+    if (intApexIndex == NO_COORD_INDEX)
         return true;
     //--- check for duplicate corner apex vertex
     if ( vertex[intApexIndex].equals2D(corner[1]) ) {
@@ -170,7 +161,7 @@ PolygonEarClipper::findIntersectingVertex(std::size_t cornerIdx, const std::arra
     std::vector<std::size_t> result;
     vertexCoordIndex.query(cornerEnv, result);
 
-    std::size_t dupApexIndex = NO_VERTEX_INDEX;
+    std::size_t dupApexIndex = NO_COORD_INDEX;
     //--- check for duplicate vertices
     for (std::size_t i = 0; i < result.size(); i++) {
         std::size_t vertIndex = result[i];
@@ -200,10 +191,10 @@ PolygonEarClipper::findIntersectingVertex(std::size_t cornerIdx, const std::arra
         else if (geom::Triangle::intersects(corner[0], corner[1], corner[2], v))
             return vertIndex;
     }
-    if (dupApexIndex != NO_VERTEX_INDEX) {
+    if (dupApexIndex != NO_COORD_INDEX) {
         return dupApexIndex;
     }
-    return NO_VERTEX_INDEX;
+    return NO_COORD_INDEX;
 }
 
 
@@ -269,7 +260,7 @@ PolygonEarClipper::removeCorner()
     }
     vertexNext[cornerIndex[0]] = vertexNext[cornerApexIndex];
     vertexCoordIndex.remove(cornerApexIndex);
-    vertexNext[cornerApexIndex] = NO_VERTEX_INDEX;
+    vertexNext[cornerApexIndex] = NO_COORD_INDEX;
     vertexSize--;
     //-- adjust following corner indexes
     cornerIndex[1] = nextIndex(cornerIndex[0]);
@@ -281,7 +272,7 @@ PolygonEarClipper::removeCorner()
 bool
 PolygonEarClipper::isRemoved(std::size_t vertexIndex) const
 {
-    return NO_VERTEX_INDEX == vertexNext[vertexIndex];
+    return NO_COORD_INDEX == vertexNext[vertexIndex];
 }
 
 
@@ -356,7 +347,7 @@ std::unique_ptr<Polygon>
 PolygonEarClipper::toGeometry() const
 {
     auto gf = geom::GeometryFactory::create();
-    std::unique_ptr<geom::CoordinateArraySequence> cs(new geom::CoordinateArraySequence());
+    std::unique_ptr<geom::CoordinateSequence> cs(new geom::CoordinateSequence());
     std::size_t index = vertexFirst;
     for (std::size_t i = 0; i < vertexSize; i++) {
         const Coordinate& v = vertex[index];
